
racial_tech_desires = {}

local function adjust(bonus, lower, upper)
  return { ..bonus, ..lower, ..upper}
end

racial_tech_desires.phidi = {
  economy       = adjust(0.3, 0.6),
  expansion     = adjust(  0, 0.3),
}

racial_tech_desires.teros = {
  carriers        = adjust(0.3, 0.3),
  carrier_bombers = adjust(0.3, 0.3),
  industry        = adjust(  0, 0.3),
}

racial_tech_desires.haduir = {
  carriers        = adjust(0.3, 0.3),
  carrier_bombers = adjust(0.3, 0.3),
  economy         = adjust(  0, 0.3),
}

racial_tech_desires.orthin = {
  missiles       = adjust(   0, nil, -0.3),
  heavy_missiles = adjust(   0, nil, -0.3),
  kinetics       = adjust(-0.3, nil, -0.3),
  heavy_kinetics = adjust(-0.3, nil, -0.3),
  armor          = adjust(-0.3),

  lasers         = adjust(0, 0.3),
  heavy_lasers   = adjust(0, 0.3),
  ion_beams      = adjust(0, 0.3),
  heavy_ions     = adjust(0, 0.3),
  siege_ions     = adjust(0, 0.3),
  shields        = adjust(0, 0.3),
  science        = adjust(0, 0.3),
}

racial_tech_desires.yoral = {
  kinetics       = adjust(0.3, 0.3),
  heavy_kinetics = adjust(0.3, 0.3),
  heavy_missiles = adjust(0.3, 0.3),
  armor          = adjust(0.3, 0.3),
  industry       = adjust(0, 0.3),
  expansion      = adjust(0, 0.3),
}

racial_tech_desires.gremak = {
  boarding_tactics = adjust(0.3, 0.3),
  battleships      = adjust(0, 0.3),
}

racial_tech_desires.tinker = {
  missiles       = adjust(0.3, 0.3),
  heavy_missiles = adjust(0.3, 0.3),
}

local function apply_adjustments(value, adjust)
  local bonus, lower, upper in adjust
  value += bonus or 0
  if lower
    value = max(value, lower)
  end
  if upper
    value = min(value, upper)
  end
  return value
end

function ensure_research_path_desires(empire)
  local research_path_desires in empire.ai_state

  if not research_path_desires
    empire.ai_state.research_path_desires = create_research_path_desires(empire, RESEARCH_PATHS)
  else
    --check if any new research paths have been added since the game was saved

    local not_found = {}
    for path ; RESEARCH_PATHS
      if not research_path_desires[path]
        not_found[path] = true
      end
    end

    if next(not_found)
      local new_desires = create_research_path_desires(empire, not_found)
      for path,value ; new_desires
        research_path_desires[path] = value
      end
    end

  end
end

function create_research_path_desires(empire, create_paths)
  local desires = {}

  --first, initialize everything to a random value between -1 and 1
  for path ; create_paths
    desires[path] = -1 + 2*random()
  end

  --note that each stage here may depend on values from the previous stage

  --next apply racial rules
  if racial_tech_desires[empire.species]
    for path, adjust ; racial_tech_desires[empire.species]
      if desires[path]
        desires[path] = apply_adjustments(desires[path], adjust)
      end
    end
  end

  --then apply any other rules
  for path, value ; desires
    --secondary paths should not be the main focus of research
    if RESEARCH_PATHS[path].flags['SECONDARY']
      desires[path] = min(value, 0.3)
    end
  end

  for path, value ; desires
    --if this path is contingent on another, cap our own desire if the contingent desire is low
    local contingent = RESEARCH_PATHS[path]?.contingent_on
    if contingent and desires[contingent] < 0
      desires[path] = min(value, desires[contingent] + 0.3)
    end
  end

  return desires
end

-- if lua_load_complete
--   for i, empire ; all_empires
--     empire.ai_state.research_path_desires = nil
--   end
-- end